@@ -18,10 +18,12 @@ import sys
 import warnings
 from . import exceptions
 from .api import library
-from .resource import increment_refcount, decrement_refcount
+from .resource import (increment_refcount, decrement_refcount, Resource,
+                       DestroyedResourceError)
+from .color import Color
 
 
-__all__ = 'FILTER_TYPES', 'Image', 'ClosedImageError'
+__all__ = 'FILTER_TYPES', 'Image', 'Iterator', 'ClosedImageError'
 
 
 #: (:class:`tuple`) The list of filter types.
@@ -52,7 +54,7 @@ FILTER_TYPES = ('undefined', 'point', 'box', 'triangle', 'hermite', 'hanning',
                 'catrom', 'mitchell', 'lanczos', 'bessel', 'sinc')
 
 
-class Image(object):
+class Image(Resource):
     """An image object.
 
     :param image: makes an exact copy of the ``image``
@@ -60,8 +62,35 @@ class Image(object):
     :param filename: opens an image of the ``filename``
     :type filename: :class:`basestring`
 
+    .. describe:: [left:right, top:bottom]
+
+       Crops the image by its ``left``, ``right``, ``top`` and ``bottom``,
+       and then returns the cropped one. ::
+
+           with img[100:200, 150:300] as cropped:
+               # manipulated the cropped image
+               pass
+
+       Like other subscriptable objects, default is 0 or its width/height::
+
+           img[:, :]        #--> just clone
+           img[:100, 200:]  #--> equivalent to img[0:100, 200:img.height]
+
+       Negative integers count from the end (width/height)::
+
+           img[-70:-50, -20:-10]
+           #--> equivalent to img[width-70:width-50, height-20:height-10]
+
+       :returns: the cropped image
+       :rtype: :class:`Image`
+
     """
 
+    c_is_resource = library.IsMagickWand
+    c_destroy_resource = library.DestroyMagickWand
+    c_get_exception = library.MagickGetException
+    c_clear_exception = library.MagickClearException
+
     __slots__ = '_wand',
 
     def __init__(self, image=None, blob=None, filename=None):
@@ -73,8 +102,7 @@ class Image(object):
                  for b in args[:i] + args[i + 1:]):
             raise TypeError('parameters are exclusive each other; use only '
                             'one at once')
-        increment_refcount()
-        try:
+        with self.allocate():
             if image is not None:
                 if not isinstance(image, Image):
                     raise TypeError('image must be a wand.image.Image '
@@ -95,9 +123,6 @@ class Image(object):
                     library.MagickReadImage(self.wand, filename)
                 else:
                     raise TypeError('invalid argument(s)')
-        except:
-            decrement_refcount()
-            raise
         self.raise_exception()
 
     @property
@@ -106,63 +131,33 @@ class Image(object):
         :exc:`ClosedImageError` when the instance has destroyed already.
 
         """
-        if self._wand is None:
+        try:
+            return self.resource
+        except DestroyedResourceError:
             raise ClosedImageError(repr(self) + ' is closed already')
-        return self._wand
 
     @wand.setter
     def wand(self, wand):
-        if library.IsMagickWand(wand):
-            self._wand = wand
-        else:
+        try:
+            self.resource = wand
+        except TypeError:
             raise TypeError(repr(wand) + ' is not a MagickWand instance')
 
     @wand.deleter
     def wand(self):
-        library.DestroyMagickWand(self.wand)
-        self._wand = None
-
-    def get_exception(self):
-        """Gets a current exception instance.
-
-        :returns: a current exception. it can be ``None`` as well if any
-                  errors aren't occurred
-        :rtype: :class:`wand.exceptions.WandException`
-
-        """
-        severity = ctypes.c_int()
-        desc = library.MagickGetException(self.wand, ctypes.byref(severity))
-        if severity.value == 0:
-            return
-        library.MagickClearException(self.wand)
-        exc_cls = exceptions.TYPE_MAP[severity.value]
-        return exc_cls(ctypes.string_at(desc))
-
-    def raise_exception(self, stacklevel=1):
-        """Raises an exception or warning if it has occurred."""
-        e = self.get_exception()
-        if isinstance(e, Warning):
-            warnings.warn(e, stacklevel=stacklevel + 1)
-        elif isinstance(e, Exception):
-            raise e
-
-    def __enter__(self):
-        return self
-
-    def __exit__(self, type, value, traceback):
-        self.close()
-
-    def __del__(self):
-        self.close()
+        del self.resource
 
     def close(self):
         """Closes the image explicitly. If you use the image object in
         :keyword:`with` statement, it was called implicitly so don't have to
         call it.
 
+        .. note::
+
+           It has the same functionality of :attr:`destroy()` method.
+
         """
-        del self.wand
-        decrement_refcount()
+        self.destroy()
 
     def clone(self):
         """Clones the image. It is equivalent to call :class:`Image` with
@@ -178,6 +173,61 @@ class Image(object):
         """
         return type(self)(image=self)
 
+    def __len__(self):
+        return self.height
+
+    def __iter__(self):
+        return Iterator(image=self)
+
+    def __getitem__(self, idx):
+        if isinstance(idx, collections.Iterable):
+            idx = tuple(idx)
+            d = len(idx)
+            if not (1 <= d <= 2):
+                raise ValueError('index cannot be {0}-dimensional'.format(d))
+            elif d == 2:
+                x, y = idx
+                x_slice = isinstance(x, slice)
+                y_slice = isinstance(y, slice)
+                if x_slice and not y_slice:
+                    y = slice(y, y + 1)
+                elif not x_slice and y_slice:
+                    x = slice(x, x + 1)
+                elif not (x_slice or y_slice):
+                    raise NotImplementedError('pixel getter is not '
+                                              'implemented yet')
+                if not (x.step is None and y.step is None):
+                    raise ValueError('slicing with step is unsupported')
+                elif (x.start is None and x.stop is None and
+                      y.start is None and y.stop is None):
+                    return self.clone()
+                def abs_(n, m, null=None):
+                    if n is None:
+                        return m if null is None else null
+                    elif not isinstance(n, numbers.Integral):
+                        raise TypeError('expected integer, not ' + repr(n))
+                    elif n > m:
+                        raise IndexError(repr(n) + ' > ' + repr(m))
+                    return m + n if n < 0 else n
+                x_start = abs_(x.start, self.width, 0)
+                x_stop = abs_(x.stop, self.width)
+                y_start = abs_(y.start, self.height, 0)
+                y_stop = abs_(y.stop, self.height)
+                width = x_stop - x_start
+                height = y_stop - y_start
+                if width < 1:
+                    raise ValueError('image width cannot be zero')
+                elif height < 1:
+                    raise ValueError('image width cannot be zero')
+                cloned = self.clone()
+                library.MagickCropImage(cloned.wand, width, height,
+                                        x_start, y_start)
+                self.raise_exception()
+                return cloned
+            else:
+                raise NotImplementedError('1d indexing is not implemented yet')
+        raise NotImplementedError('row getter is not implemented yet')
+
     @property
     def width(self):
         """(:class:`numbers.Integral`) The width of this image."""
@@ -279,7 +329,77 @@ class Image(object):
         return blob
 
 
-class ClosedImageError(ReferenceError, AttributeError):
+class Iterator(Resource, collections.Iterator):
+    """Row iterator for :class:`Image`. It shouldn't be instantiated
+    directly; instead, it can be acquired through :class:`Image` instance::
+
+        assert isinstance(image, wand.image.Image)
+        iterator = iter(image)
+
+    It doesn't iterate every pixel, but rows. For example::
+
+        for row in image:
+            for col in row:
+                assert isinstance(col, wand.color.Color)
+                print col
+
+    Every row is a :class:`collections.Sequence` which consists of
+    one or more :class:`wand.color.Color` values.
+
+    :param image: the image to get an iterator
+    :type image: :class:`Image`
+
+    """
+
+    c_is_resource = library.IsPixelIterator
+    c_destroy_resource = library.DestroyPixelIterator
+    c_get_exception = library.PixelGetIteratorException
+    c_clear_exception = library.PixelClearIteratorException
+
+    def __init__(self, image=None, iterator=None):
+        if image is not None and iterator is not None:
+            raise TypeError('it takes only one argument at a time')
+        with self.allocate():
+            if image is not None:
+                if not isinstance(image, Image):
+                    raise TypeError('expected a wand.image.Image instance, '
+                                    'not ' + repr(image))
+                self.resource = library.NewPixelIterator(image.wand)
+                self.height = image.height
+            else:
+                if not isinstance(iterator, Iterator):
+                    raise TypeError('expected a wand.image.Iterator instance, '
+                                    'not ' + repr(iterator))
+                self.resource = library.ClonePixelIterator(iterator.resource)
+                self.height = iterator.height
+        self.raise_exception()
+        self.cursor = 0
+
+    def __iter__(self):
+        return self
+
+    def next(self, x=None):
+        if self.cursor >= self.height:
+            self.destroy()
+            raise StopIteration()
+        self.cursor += 1
+        width = ctypes.c_size_t()
+        f = library.PixelGetNextIteratorRow
+        f.restype = ctypes.POINTER(ctypes.c_void_p)
+        pixels = f(self.resource, ctypes.byref(width))
+        get_color = library.PixelGetColorAsString
+        get_color.restype = ctypes.c_char_p
+        if x is None:
+            pixels = [Color(get_color(pixels[x])) for x in xrange(width.value)]
+            return pixels
+        return Color(get_color(pixels[x]))
+
+    def clone(self):
+        """Clones the same iterator."""
+        return type(self)(iterator=self)
+
+
+class ClosedImageError(DestroyedResourceError):
     """An error that rises when some code tries access to an already closed
     image.
 
